//+------------------------------------------------------------------+
//|                                                   Swing line.mq4 |
//|                                                                  |
//| September 2010 issue                                             |
//| Technical Analysis of Stocks and Commodities                     |
//| Ron Black's article Getting Clear With Short-Term Swings       |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers  2
#property indicator_color1   clrDeepSkyBlue
#property indicator_color2   clrSandyBrown
#property indicator_width1   2
#property indicator_width2   2
#property indicator_minimum  0
#property indicator_maximum  1

//
//
//
//
//

extern ENUM_TIMEFRAMES TimeFrame   = PERIOD_CURRENT;
extern bool   alertsOn             = false;
extern bool   alertsOnCurrent      = true;
extern bool   alertsMessage        = true;
extern bool   alertsSound          = false;
extern bool   alertsEmail          = false;
extern bool   verticalLinesVisible = false;
extern string verticalLinesID      = "swLines";
extern color  verticalLinesUpColor = clrDeepSkyBlue;
extern color  verticalLinesDnColor = clrSandyBrown;
extern int    verticalLinesStyle   = STYLE_DOT;
extern int    verticalLinesWidth   = 0;

//
//
//
//
//

double swli[];
double histou[];
double histod[];
double trend[];
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(4); 
   SetIndexBuffer(0,histou); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,histod); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,swli);
   SetIndexBuffer(3,trend);

   //
   //
   //
   //
   //
   
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = MathMax(TimeFrame,_Period);
   IndicatorShortName("Swing line "+timeFrameToString(TimeFrame));
   return(0);
}
int deinit() 
{ 
   string lookFor       = verticalLinesID+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0); 
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double  work[][4];
#define hHi 0
#define hLo 1
#define lHi 2
#define lLo 3

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { histou[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (TimeFrame==Period())
   {
      if (ArrayRange(work,0)!=Bars) ArrayResize(work,Bars);

      //
      //
      //
      //
      //
      
      for(int i=limit,r=Bars-limit-1; i>=0; i--,r++)
      {
         if (i==Bars-1)
         {
            work[r][hHi] = High[i]; work[r][hLo] = Low[i];
            work[r][lHi] = High[i]; work[r][lLo] = Low[i];
            trend[i]     = -1;
            continue; 
         }
      
         //
         //
         //
         //
         //
      
         trend[i]     = trend[i+1];
         work[r][hHi] = work[r-1][hHi]; work[r][hLo] = work[r-1][hLo];
         work[r][lHi] = work[r-1][lHi]; work[r][lLo] = work[r-1][lLo];
 
               if (trend[i+1] == 1)
               {
                  work[r][hHi] = MathMax(work[r-1][hHi],High[i]);
                  work[r][hLo] = MathMax(work[r-1][hLo],Low[i]);
                     if (High[i]<work[r][hLo]) { trend[i] = -1; work[r][lHi] = High[i]; work[r][lLo] = Low[i]; }
               }
               if (trend[i+1] == -1)
               {
                  work[r][lHi] = MathMin(work[r-1][lHi],High[i]);
                  work[r][lLo] = MathMin(work[r-1][lLo],Low[i]);
                     if (Low[i]>work[r][lHi]) { trend[i] =  1; work[r][hHi] = High[i]; work[r][hLo] = Low[i]; }
               }

         if (trend[i]==1)      
               swli[i] = work[r][hLo];
         else  swli[i] = work[r][lHi];
         
         histou[i] = EMPTY_VALUE; histod[i] = EMPTY_VALUE; if (trend[i] == 1) histou[i] = 1; if (trend[i] == -1) histod[i] = 1;
         if (verticalLinesVisible)
         {
            ObjectDelete(verticalLinesID+":"+Time[i]);
            if (trend[i]!=trend[i+1])
            {
              if (trend[i] == 1) drawLine(i,verticalLinesUpColor);
              if (trend[i] ==-1) drawLine(i,verticalLinesDnColor);
            }
         }      
      }
      manageAlerts();
      return(0);
   }            
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         histou[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,verticalLinesVisible,verticalLinesID,verticalLinesUpColor,verticalLinesDnColor,verticalLinesStyle,verticalLinesWidth,0,y);
         histod[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,verticalLinesVisible,verticalLinesID,verticalLinesUpColor,verticalLinesDnColor,verticalLinesStyle,verticalLinesWidth,1,y);
   }
   return(0);
}



//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"up");
         if (trend[whichBar] ==-1) doAlert(whichBar,"down");
      }         
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," "+timeFrameToString(_Period)+" Swing line changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"Swing line "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
string StringUpperCase(string str) { string s = str; StringToUpper(s); return(s); }

//
//
//
//
//

void drawLine(int i,color theColor)
{
      string name = verticalLinesID+":"+Time[i];
   
      //
      //
      //
      //
      //
      
      int add = 0;
      ObjectCreate(name,OBJ_VLINE,0,Time[i]+add,0);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_STYLE,verticalLinesStyle);
         ObjectSet(name,OBJPROP_WIDTH,verticalLinesWidth);
         ObjectSet(name,OBJPROP_BACK,true);
}